package com.chinacreator.ecd.flowstatus.controller;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.ecd.common.BusinessException;
import com.chinacreator.ecd.common.ErrorInfo;
import com.chinacreator.ecd.common.service.InterfaceUserService;
import com.chinacreator.ecd.flowstatus.service.FlowstatusService;
import com.chinacreator.ecd.util.DESUtil;
import com.chinacreator.ecd.util.IpAddrUtil;
import com.chinacreator.ecd.util.URLUtil;

@RestController
public class FlowstatusController {
	
	private Logger logger = LoggerFactory.getLogger(FlowstatusController.class);
	
	@Autowired
	private InterfaceUserService interfaceUserService;
	
	@Autowired
	private FlowstatusService flowstatusService;
	
	// 接口数据解密密码
	private final static String PASSWORD = "dy#@24fL";
	
	@RequestMapping("/acceptFlowstatus")
	public String acceptFlowstatus(HttpServletRequest request, HttpServletResponse response) throws IOException, Exception {
		long starttime = System.currentTimeMillis();
		
		String requestIp = IpAddrUtil.getIpAddr(request);
		String dataString = URLUtil.readInputStream(request.getInputStream());
		JSONObject data = null;
		JSONObject result = new JSONObject();
		result.put("resultcode", "0");
		result.put("errorinfo", "ok");
		
		try {
			data = JSON.parseObject(dataString);
			String timestamp = data.getString("timestamp");
			String params = data.getString("params");
			String signature = data.getString("signature");
			
			// interfaceUserService.validateIp("Floor8Kafka", "acceptFlowstatus", requestIp);
			interfaceUserService.validateTimestamp("Floor8Kafka", "acceptFlowstatus", timestamp);
			interfaceUserService.validateSignature("Floor8Kafka", "acceptFlowstatus", params, timestamp, signature);
			
			params = DESUtil.decode(params, PASSWORD);
			JSONObject jsonParams = JSONObject.parseObject(params);
			
			data.put("params", jsonParams);
			validateParam(data);
			
			// 将数据推送到kafka
			flowstatusService.sendFlowuseToKafka(jsonParams);
			
			long endtime = System.currentTimeMillis();
			
			logger.info("保存到Kafka成功={}, ip={}，用时={}ms", data.toJSONString(), requestIp, endtime - starttime);
			
		} catch (BusinessException e) {
			
			result.put("resultcode", e.getResultcode());
			result.put("errorinfo", e.getErrorinfo());
			
			long endtime = System.currentTimeMillis();
			
			logger.error("保存到Kafka失败={}, 原因={},{}, ip={}, agent={}，用时={}ms",
					data.toJSONString(), e.getResultcode(), e.getErrorinfo(), requestIp, IpAddrUtil.getUserAgent(request), endtime - starttime);
		} catch (Exception e) {
			result.put("resultcode", "8000");
			result.put("errorinfo", e.getMessage());
			
			long endtime = System.currentTimeMillis();
			
			logger.error("保存到Kafka失败={}, 原因={}, ip={}, agent={}，用时={}ms",
					data.toJSONString(), e.getMessage(), requestIp, IpAddrUtil.getUserAgent(request), endtime - starttime);
		}
		
		return result.toJSONString();
	}
	
	private static void validateParam(JSONObject data) {
		if (data == null) {
			throw new BusinessException(ErrorInfo.code0009, ErrorInfo.errorDesc.get(ErrorInfo.code0009));
		}
	}
	
}
