package com.chinacreator.ecd.util;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;

public class IpAddrUtil {

	public static String getIpAddr(HttpServletRequest request) {
		String ip = request.getHeader("X-Real-IP");
		if (inValidateIpAddr(ip)) {
			ip = request.getHeader("X-Forwarded-For");
		}
		if (inValidateIpAddr(ip)) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		if (inValidateIpAddr(ip)) {
			ip = request.getHeader("WL-Proxy-Client-IP");
		}
		if (inValidateIpAddr(ip)) {
			ip = request.getHeader("clientip");
		}
		if (inValidateIpAddr(ip)) {
			ip = request.getRemoteAddr();
		}
		if (inValidateIpAddr(ip)) {
			ip = "null";
		} else {
			int index = ip.indexOf(",");
			if (index != -1) {
				ip = ip.substring(0, index);
			}
		}
		return ip;
	}

	/**
	 * ip地址是否有效
	 * 114.255.201.224 是8楼的一个外网IP
	 * 100.0.x.x 是9楼的内网的网段
	 * @param ip
	 * @return true：无效，false有效
	 */
	private static boolean inValidateIpAddr(String ip) {
		return ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip) || ip.startsWith("10.")
				|| ip.startsWith("172.") || ip.startsWith("192.") || ip.startsWith("100.0.");
	}

	public static String getUserAgent(HttpServletRequest request) {
		return request.getHeader("User-Agent");
	}

	public static String getHostIp() {
		try {
			Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
			while (en.hasMoreElements()) {
				NetworkInterface intf = (NetworkInterface) en.nextElement();
				Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses();
				while (enumIpAddr.hasMoreElements()) {
					InetAddress inetAddress = (InetAddress) enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress() && !inetAddress.isLinkLocalAddress() && inetAddress.isSiteLocalAddress()) {
						return inetAddress.getHostAddress().toString();
					}
				}
			}
		} catch (SocketException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static boolean isBlockIP(String blockIP, String remoteIp) {
		if (blockIP.indexOf(remoteIp) >= 0) {
			return true;
		}

		return false;
	}
}
